@@ -1,35 +1,48 @@
 use std::{future::Future, io, pin::Pin, task::Context, task::Poll};
 
-pub use open_ssl::ssl::{Error as SslError, HandshakeError, SslConnector, SslMethod};
-pub use tokio_openssl::SslStream;
+pub use ntex_tls::openssl::SslFilter;
+pub use tls_openssl::ssl::{Error as SslError, HandshakeError, SslConnector, SslMethod};
 
-use crate::rt::net::TcpStream;
+use ntex_tls::openssl::SslConnector as IoSslConnector;
+
+use crate::io::{utils::Boxed, Base, Io};
 use crate::service::{Service, ServiceFactory};
-use crate::util::Ready;
+use crate::util::{PoolId, Ready};
 
-use super::{Address, Connect, ConnectError, Connector};
+use super::{Address, Connect, ConnectError, Connector as BaseConnector};
 
-pub struct OpensslConnector<T> {
-    connector: Connector<T>,
+pub struct Connector<T> {
+    connector: BaseConnector<T>,
     openssl: SslConnector,
 }
 
-impl<T> OpensslConnector<T> {
+impl<T> Connector<T> {
     /// Construct new OpensslConnectService factory
     pub fn new(connector: SslConnector) -> Self {
-        OpensslConnector {
-            connector: Connector::default(),
+        Connector {
+            connector: BaseConnector::default(),
             openssl: connector,
         }
     }
+
+    /// Set memory pool.
+    ///
+    /// Use specified memory pool for memory allocations. By default P0
+    /// memory pool is used.
+    pub fn memory_pool(self, id: PoolId) -> Self {
+        Self {
+            connector: self.connector.memory_pool(id),
+            openssl: self.openssl,
+        }
+    }
 }
 
-impl<T: Address + 'static> OpensslConnector<T> {
+impl<T: Address + 'static> Connector<T> {
     /// Resolve and connect to remote host
     pub fn connect<U>(
         &self,
         message: U,
-    ) -> impl Future<Output = Result<SslStream<TcpStream>, ConnectError>>
+    ) -> impl Future<Output = Result<Io<SslFilter<Base>>, ConnectError>>
     where
         Connect<T>: From<U>,
     {
@@ -45,13 +58,11 @@ impl<T: Address + 'static> OpensslConnector<T> {
             match openssl.configure() {
                 Err(e) => Err(io::Error::new(io::ErrorKind::Other, e).into()),
                 Ok(config) => {
-                    let config = config
+                    let ssl = config
                         .into_ssl(&host)
                         .map_err(|e| io::Error::new(io::ErrorKind::Other, e))?;
-                    let mut io = SslStream::new(config, io)
-                        .map_err(|e| io::Error::new(io::ErrorKind::Other, e))?;
-                    match Pin::new(&mut io).connect().await {
-                        Ok(_) => {
+                    match io.add_filter(IoSslConnector::new(ssl)).await {
+                        Ok(io) => {
                             trace!("SSL Handshake success: {:?}", host);
                             Ok(io)
                         }
@@ -65,34 +76,37 @@ impl<T: Address + 'static> OpensslConnector<T> {
             }
         }
     }
+
+    /// Produce sealed io stream (IoBoxed)
+    pub fn seal(self) -> Boxed<Connector<T>, Connect<T>> {
+        Boxed::new(self)
+    }
 }
 
-impl<T> Clone for OpensslConnector<T> {
+impl<T> Clone for Connector<T> {
     fn clone(&self) -> Self {
-        OpensslConnector {
+        Connector {
             connector: self.connector.clone(),
             openssl: self.openssl.clone(),
         }
     }
 }
 
-impl<T: Address + 'static> ServiceFactory for OpensslConnector<T> {
-    type Request = Connect<T>;
-    type Response = SslStream<TcpStream>;
+impl<T: Address, C> ServiceFactory<Connect<T>, C> for Connector<T> {
+    type Response = Io<SslFilter<Base>>;
     type Error = ConnectError;
-    type Config = ();
-    type Service = OpensslConnector<T>;
+    type Service = Connector<T>;
     type InitError = ();
     type Future = Ready<Self::Service, Self::InitError>;
 
-    fn new_service(&self, _: ()) -> Self::Future {
+    #[inline]
+    fn new_service(&self, _: C) -> Self::Future {
         Ready::Ok(self.clone())
     }
 }
 
-impl<T: Address + 'static> Service for OpensslConnector<T> {
-    type Request = Connect<T>;
-    type Response = SslStream<TcpStream>;
+impl<T: Address> Service<Connect<T>> for Connector<T> {
+    type Response = Io<SslFilter<Base>>;
     type Error = ConnectError;
     type Future = Pin<Box<dyn Future<Output = Result<Self::Response, Self::Error>>>>;
 
@@ -101,6 +115,7 @@ impl<T: Address + 'static> Service for OpensslConnector<T> {
         Poll::Ready(Ok(()))
     }
 
+    #[inline]
     fn call(&self, req: Connect<T>) -> Self::Future {
         Box::pin(self.connect(req))
     }
@@ -118,7 +133,7 @@ mod tests {
         });
 
         let ssl = SslConnector::builder(SslMethod::tls()).unwrap();
-        let factory = OpensslConnector::new(ssl.build()).clone();
+        let factory = Connector::new(ssl.build()).clone();
 
         let srv = factory.new_service(()).await.unwrap();
         let result = srv
