@@ -36,7 +36,6 @@ pub(crate) use ntex_macros::rt_test2 as rt_test;
 pub mod connect;
 pub mod http;
 pub mod server;
-pub mod util;
 pub mod web;
 pub mod ws;
 
@@ -64,6 +63,12 @@ pub mod router {
 pub mod rt {
     //! A runtime implementation that runs everything on the current thread.
     pub use ntex_rt::*;
+
+    #[cfg(feature = "tokio")]
+    pub use ntex_tokio::*;
+
+    #[cfg(all(not(feature = "tokio"), feature = "async-std"))]
+    pub use ntex_async_std::*;
 }
 
 pub mod service {
@@ -78,6 +83,8 @@ pub mod time {
 pub mod io {
     //! IO streaming utilities.
     pub use ntex_io::*;
+
+    pub use ntex_tokio::TokioIoBoxed;
 }
 
 pub mod testing {
@@ -91,3 +98,8 @@ pub mod tls {
     //! TLS support for ntex ecosystem.
     pub use ntex_tls::*;
 }
+
+pub mod util {
+    pub use ntex_bytes::{Buf, BufMut, ByteString, Bytes, BytesMut, Pool, PoolId, PoolRef};
+    pub use ntex_util::{future::*, ready, services::*, HashMap, HashSet};
+}
