@@ -3,9 +3,9 @@ use std::cell::Cell;
 use crate::codec::{Decoder, Encoder};
 use crate::util::{ByteString, Bytes, BytesMut};
 
+use super::error::ProtocolError;
 use super::frame::Parser;
 use super::proto::{CloseReason, OpCode};
-use super::ProtocolError;
 
 /// WebSocket message
 #[derive(Debug, PartialEq)]
@@ -62,6 +62,7 @@ bitflags::bitflags! {
         const SERVER         = 0b0000_0001;
         const R_CONTINUATION = 0b0000_0010;
         const W_CONTINUATION = 0b0000_0100;
+        const CLOSED         = 0b0000_1000;
     }
 }
 
@@ -90,6 +91,11 @@ impl Codec {
         self
     }
 
+    /// Check if codec encoded `Close` message
+    pub fn is_closed(&self) -> bool {
+        self.flags.get().contains(Flags::CLOSED)
+    }
+
     fn insert_flags(&self, f: Flags) {
         let mut flags = self.flags.get();
         flags.insert(f);
@@ -143,11 +149,10 @@ impl Encoder for Codec {
                 true,
                 !self.flags.get().contains(Flags::SERVER),
             ),
-            Message::Close(reason) => Parser::write_close(
-                dst,
-                reason,
-                !self.flags.get().contains(Flags::SERVER),
-            ),
+            Message::Close(reason) => {
+                self.insert_flags(Flags::CLOSED);
+                Parser::write_close(dst, reason, !self.flags.get().contains(Flags::SERVER))
+            }
             Message::Continuation(cont) => match cont {
                 Item::FirstText(data) => {
                     if self.flags.get().contains(Flags::W_CONTINUATION) {
@@ -215,8 +220,7 @@ impl Decoder for Codec {
     type Error = ProtocolError;
 
     fn decode(&self, src: &mut BytesMut) -> Result<Option<Self::Item>, Self::Error> {
-        match Parser::parse(src, self.flags.get().contains(Flags::SERVER), self.max_size)
-        {
+        match Parser::parse(src, self.flags.get().contains(Flags::SERVER), self.max_size) {
             Ok(Some((finished, opcode, payload))) => {
                 // handle continuation
                 if !finished {
